/// <binding />
'use strict';

var gulp = require('gulp')
  , glob = require('glob')
  , chokidar = require('chokidar')
  , webpack = require("webpack")
  , namedModulesPlugin = require('webpack/lib/NamedModulesPlugin')
  , gutil = require("gulp-util")
  , merge = require('webpack-merge')
  , exec = require('child_process').exec
  , fs = require('fs')
  , fsExtra = require('fs-extra')
  , argv = require('yargs').argv
  , del = require('del')
  , $ = require('gulp-load-plugins')({
      pattern: [
        'gulp-*'
      ],
      rename: {
      }
  })
, injectString = require('gulp-inject-string')
, timestamp = require('console-timestamp')
, rollup = require('rollup')
, rollupBabel = require('rollup-plugin-babel')
, rollupCommonjs = require('rollup-plugin-commonjs')
, rollupNodeResolve = require('rollup-plugin-node-resolve')
, path = require('path')
, exec = require('child_process').exec
, omniaPlugins = require('./TaskRunner/omf.webpack.plugins.js')
, utils = require('./TaskRunner/omf.utils.js');

var appConfig = require('./TaskRunner/config/app.json');
mergeConfig();
//require('require-dir')('./TaskRunner/tasks');

gulp.task('build', function (cb) {
    aotCompile()
        .then(webpackBundle)
        .then(function () { console.log(timestamp('[hh:mm:ss]') + ' Build finished'); })
});

gulp.task('aot-compile', function (cb) {
    aotCompile().then(cb);
})

gulp.task('webpack-bundle', function (cb) {
    webpackBundle(true).then(cb);
});

gulp.task('requirejs-lib', function () {
    return $.requirejs({
        baseUrl: 'Scripts/angular/omf.angular.config.js',
        out: 'angular.lib.js',
        shim: {
            // standard require.js shim options 
        },
        // ... more require.js options 
    })
    .pipe(gulp.dest('.')); // pipe it to the output DIR 
});

gulp.task('rollup-angular-lib', function (cb) {
    rollup.rollup({
        entry: 'Scripts/angular/omf.angular.config.js',
        dest: 'rollup-bundle.js', // output a single application bundle
        sourceMap: false,
        format: 'iife',
        onwarn: function (warning) {
            // Skip certain warnings

            // should intercept ... but doesn't in some rollup versions
            if (warning.code === 'THIS_IS_UNDEFINED') { return; }

            // console.warn everything else
            console.warn(warning.message);
        },
        plugins: [
            rollupNodeResolve({ jsnext: true, module: true }),
            rollupCommonjs({
                include: 'node_modules/rxjs/**',
            }),
            //uglify()
        ]
        //// tell rollup our main entry point
        //entry: 'Scripts/angular/omf.angular.config.js',
        //plugins: [
        //    // configure rollup-babel to use the ES2015 Rollup preset
        //    // and not transpile any node_modules files
        //    rollupBabel({
        //        exclude: 'node_modules/**',
        //        presets: 'es2015-rollup'
        //    }),
        //    // minify with uglify
        //   // uglify()
        //]
    }).then((bundle) => {
        console.log('Bundle start');

        // write bundle to a file and use the IIFE format so it executes immediately
        return bundle.write({
            format: 'iife',
            dest: './rollup-bundle.js'
        });
    }).then(() => {
        console.log('Bundle created');
        cb();
    });
});

gulp.task('webpack-bundle-angular-libs', function (callback) {
    //fsExtra.copySync('TaskRunner/external/tools.js', 'node_modules/@angular/platform-browser/src/browser/tools/tools.js');

    webpack(merge(require("./TaskRunner/config/webpack.config.js"), {
        entry: {
            'vendor': appConfig.angular.bundle.mainfile,
            'app': './TaskRunner/config/external.js'
        },
        target: "web",
        output: {
            //libraryTarget: "amd",
         //   path: '.',
            filename: 'TaskRunner/temp/app.bundle.js',
        },
        module: {
            loaders: [
              {
                  test: /\.js$/,
                  loader: 'babel-loader',
                  query: {
                      compact: false,
                      presets: ['es2015'],
                      //plugins: ['transform-runtime'],
                      ignore: [
                        "./**/Scripts/angular/libs/zone.custom.js",
                        "./**/@angular/compiler/index.js"
                      ]
                  }
              }
            ]
        },
        plugins: [
            //new webpack.NoErrorsPlugin(),
            //new webpack.optimize.DedupePlugin(),
            new webpack.optimize.CommonsChunkPlugin({ name: 'vendor', filename: appConfig.angular.bundle.export }),
            new omniaPlugins.NamedModulesPlugin({
                replacements: [
                    {
                        pattern: /\/node_modules/g,
                        replace: ""
                    }
                ]
            }),
            new omniaPlugins.ReplaceBundlePlugin([
                {
                    partten: /function webpackJsonpCallback[\s\S]+__webpack_require__.s = executeModules\[i\]/g,
                    replacement: function (match) {
                        return  buildWebpackJsonpCallbackCustom();
                    }
                },
                {
                    partten: new RegExp("function __webpack_require__\\(moduleId\\) {", 'g'),
                    replacement: function (match) {
                        return buildWebpackResolveModuleCustom();
                    }
                },
                {
                    partten: /return __webpack_require__(.*?)omf.angular.config.js"\);/g,
                    replacement: function (match) {
                        return "";
                    }
                },
                {
                    partten: /@angular\/(.*?).es5.js/g,
                    replacement: function (match) {
                        return match.replace('.es5.js', '.js');
                    }
                },
                {
                    partten: /@angular\/core\/@angular\/core/g,
                    replacement: '@angular/core'
                },
                {
                    partten: /@angular\/platform-browser-dynamic\/@angular\/platform-browser-dynamic/g,
                    replacement: '@angular/platform-browser-dynamic'
                },
                {
                    partten: /@angular\/platform-browser\/@angular\/platform-browser/g,
                    replacement: '@angular/platform-browser'
                },
                {
                    partten: /@angular\/forms\/@angular\/forms/g,
                    replacement: '@angular/forms'
                },
                {
                    partten: /@angular\/common\/@angular\/common/g,
                    replacement: '@angular/common'
                },
                {
                    partten: /@angular\/http\/@angular\/http/g,
                    replacement: '@angular/http'
                },
                {
                    partten: /@angular\/router\/@angular\/router/g,
                    replacement: '@angular/router'
                },
                {
                    partten: /@angular\/upgrade\/@angular\/upgrade/g,
                    replacement: '@angular/upgrade'
                },
                {
                    partten: /@angular\/animations\/@angular\/animations/g,
                    replacement: '@angular/animations'
                },
                {
                    partten: /@angular\/compiler\/@angular\/compiler/g,
                    replacement: '@angular/compiler'
                }
            ])
        ]
    }),
    function (err, stats) {
        //console.log("webpack callback : " + stats);
        if (err) throw new gutil.PluginError("webpack", err);
        //gutil.log("[webpack]", stats.toString({
        //}));
        callback();
    })
    .plugin("done", function () {
        gulp.src(appConfig.angular.concat)
        .pipe($.concat(appConfig.angular.outFileName))
        .pipe(gulp.dest(appConfig.angular.outDir))
        //.pipe($.uglifyjs(appConfig.angular.outFileName, {
        //    mangle: { keep_fnames: true }
        //}))
        //.pipe(gulp.dest(appConfig.angular.outDir))
        .on('end', function () {
            console.log('bundle completed');
        });
    });
});

gulp.task('less', function () {
    console.log(utils.getCurrentDateTime() + " : compiling less files");
    compileAllLess();
    console.log(utils.getCurrentDateTime() + " : done compile less files");
});

gulp.task('watch-less', function () {
    process.on('uncaughtException', function (err) {
        console.log(err);
    });

    var watchLessTenantResource = chokidar.watch(appConfig.less.watch, {
        ignored: /[\/\\]\./,
        persistent: true
    });

    watchLessTenantResource.on('ready', function () {
        watchLessTenantResource
            .on('change', function (path) {
                console.log(utils.getCurrentDateTime() + " : compiling > " + path);
                compileAllLess();
                console.log(utils.getCurrentDateTime() + " : done");
            });
    });
});

gulp.task('webpack-watch', function () {
    var listFilesBundling = [];
    process.on('uncaughtException', function (err) {
        console.log(err);
    });

    var watchAngular = chokidar.watch(appConfig.webpack.watch, {
        ignored: /[\/\\]\./,
        persistent: true
    });

    watchAngular.on('ready', function () {
        watchAngular
            .on('change', function (filePath) {
                if (listFilesBundling.indexOf(filePath) === -1) {
                    listFilesBundling.push(filePath);
                    webpackCompile(appConfig.webpack.basePath, appConfig.webpack.ignoreFiles, filePath, function () {
                        if (listFilesBundling.indexOf(filePath) > -1)
                            listFilesBundling.splice(listFilesBundling.indexOf(filePath), 1)
                    }, true);
                }
            });
    });
});

gulp.task('watch-tenantresources-to-db', function () {
    process.on('uncaughtException', function (err) {
        listLessFileCompiling = [];
        listChangedFiles = [];
        console.log(err);
    });

    var listLessFileCompiling = [];
    var listChangedFiles = [];
    var basePath = process.cwd().replace("\gulpfile.js", "");

    var watchLessTenantResource = chokidar.watch(appConfig.less.watch, {
        ignored: /[\/\\]\./,
        persistent: true
    });

    var watchTenantResource = chokidar.watch(appConfig.omniaFoundation.watch, {
        ignored: /[\/\\]\./,
        persistent: true
    });

    watchTenantResource.on('ready', function () {
        watchTenantResource
        .on('change', function (filePath) {
            var filePathFOrmatOS = filePath.replace(/\//g, "\\");
            filePathFOrmatOS = path.join(basePath, filePathFOrmatOS);
            if (listChangedFiles.indexOf(filePathFOrmatOS) === -1) {
                listChangedFiles.push(filePathFOrmatOS);
            }
            utils.timeWatch("updateTenantResource", updateTenantResource, 500);
        });
    });


    watchLessTenantResource.on('ready', function () {
        watchLessTenantResource
            .on('change', function (path) {
                for (var i = 0; i < appConfig.less.compile.tenantResource.files.length; i++) {
                    glob(appConfig.less.compile.tenantResource.files[i], function (err, files) {
                        for (var k = 0; k < files.length; k++) {
                            compileLessTenantResource(files[k]);
                        }
                    });
                }
            });
    });

    function updateTenantResource() {
        if (listLessFileCompiling.length > 0) {
            utils.timeWatch("updateTenantResource", updateTenantResource, 500);
        }
        else {
            try {
                console.log(utils.getCurrentDateTime() + " : updating to DB");
                var cloneListChangedFiles = listChangedFiles.slice(0);
                listChangedFiles.splice(0, cloneListChangedFiles.length);
                resourceservice.updateTenantResource(cloneListChangedFiles, appConfig.omniaFoundation.updateTenantResource);
            } catch (err) {
                listChangedFiles = [];
                console.log(utils.getCurrentDateTime() + " : " + err);
            }
        }
    }

    function compileLessTenantResource(filePath) {
        var filePathFOrmatOS = filePath.replace(/\//g, "\\");
        listLessFileCompiling.push(filePathFOrmatOS);

        gulp.src(filePath)
            .pipe($.sourcemaps.init())
            .pipe($.less({}))
            .pipe($.sourcemaps.write('.'))
            .pipe(gulp.dest(filePath.substring(0, filePath.lastIndexOf('/'))))
            .on('error', function (err) {
                if (listLessFileCompiling.indexOf(filePathFOrmatOS) !== -1) {
                    listLessFileCompiling.splice(listLessFileCompiling.indexOf(filePathFOrmatOS), 1);
                }
                console.log(filePathFOrmatOS + " : " + err);
            })
            .on('end', function () {
                if (listLessFileCompiling.indexOf(filePathFOrmatOS) !== -1) {
                    listLessFileCompiling.splice(listLessFileCompiling.indexOf(filePathFOrmatOS), 1);
                }
                //console.log(filePathFOrmatOS + " : done compile less - " + utils.getCurrentDateTime());
            });
    }
});

gulp.task('build-npm-package', function () {
    gulp.src([
        'TenantResources/angular/omnia/foundation/extensibility/**/*.d.ts',
        'TenantResources/angular/omnia/foundation/extensibility/**/*.html',
        'TenantResources/angular/omnia/foundation/extensibility/**/*.metadata.json'])
        .pipe(injectString.replace('../models', '../../models'))
        .pipe(gulp.dest('SetupNpmPackage/NpmPackage/extensibility/typings'))
    gulp.src([
        'TenantResources/angular/omnia/foundation/models/**/*.d.ts'])
        .pipe(gulp.dest('SetupNpmPackage/NpmPackage/models'))
});

function aotCompile() {
    return new Promise(function (resolve, reject) {
        console.log(timestamp('[hh:mm:ss]') + ' Aot compile running...');
        del.sync('node_modules/@types/node');

        if (!enableRunAot() && (process.argv.length === 0 || process.argv[process.argv.length - 1] !== '--force')) {
            console.warn('Aot compile disabled in environment.json');
            resolve();
            return;
        }
        if (appConfig.aot.cleanBefore && appConfig.aot.cleanBefore.length > 0) {
            del.sync(appConfig.aot.cleanBefore);
        }
        exec('ngc -p ./tsconfig.aot.json', function (err, stdout, stderr) {
            if (stdout)
                console.log(stdout);
            if (stderr)
                console.log(stderr);
            if (err !== null) {
                resolve(err);
                return;
            }
            //doWebpackCompile(resolve);
            else {
                if (appConfig.aot.compileAotTsFiles && appConfig.aot.compileAotTsFiles.length > 0) {
                    console.log('Compiling aot ts files');
                    gulp.src(['TaskRunner/omf.aot.d.ts', 'Scripts/typings/angular2/globals/core-js/index.d.ts'].concat(appConfig.aot.compileAotTsFiles))
                        .pipe($.tsc({
                            "target": "es5",
                            "module": "commonjs",
                            "moduleResolution": "node",
                            "declaration": false,
                            "sourceMap": false,
                            "removeComments": false,
                            "noImplicitAny": false,
                            "emitDecoratorMetadata": true,
                            "experimentalDecorators": true,
                            "types":[]
                        }))
                        .pipe(gulp.dest('.'))
                        .on('end', function () {
                            doWebpackCompile(resolve);
                        });
                }
                else {
                    doWebpackCompile(resolve);
                }
            }
        });
    })

    function doWebpackCompile(resolve) {
        if (appConfig.aot.webpackCompileFiles && appConfig.aot.webpackCompileFiles.length > 0) {
            var totalFiles = 0;
            var countLoop = 0;
            for (var i = 0; i < appConfig.aot.webpackCompileFiles.length; i++) {
                glob(appConfig.aot.webpackCompileFiles[i], function (err, files) {
                    countLoop++;
                    totalFiles = totalFiles + files.length;
                    if (totalFiles === 0 && countLoop === appConfig.aot.webpackCompileFiles.length) {
                        console.log(timestamp('[hh:mm:ss]') + ' Aot compile finished');
                        resolve();
                    }
                    for (var k = 0; k < files.length; k++) {
                        webpackCompile(appConfig.webpack.basePaths, appConfig.webpack.ignoreFiles, files[k], function () {
                            totalFiles = totalFiles - 1;
                            if (totalFiles === 0) {
                                if (appConfig.aot.copyOutputAotFilesToTenantResources && appConfig.aot.copyOutputAotFilesToTenantResources.length > 0) {
                                    var count = 0;
                                    for (var j = 0; j < appConfig.aot.copyOutputAotFilesToTenantResources.length; j++) {
                                        gulp.src(appConfig.aot.copyOutputAotFilesToTenantResources[j].source)
                                            .pipe(gulp.dest(appConfig.aot.copyOutputAotFilesToTenantResources[j].target))
                                            .on('end', function () {
                                                count++;
                                                if (count === appConfig.aot.copyOutputAotFilesToTenantResources.length) {
                                                    if (appConfig.aot.cleanAfter && appConfig.aot.cleanAfter.length > 0) {
                                                        del.sync(appConfig.aot.cleanAfter);
                                                    }
                                                    console.log(timestamp('[hh:mm:ss]') + ' Aot compile finished');
                                                    resolve();
                                                }
                                            });
                                    }
                                }
                                else {
                                    if (appConfig.aot.cleanAfter && appConfig.aot.cleanAfter.length > 0) {
                                        del.sync(appConfig.aot.cleanAfter);
                                    }
                                    console.log(timestamp('[hh:mm:ss]') + ' Aot compile finished');
                                    resolve();
                                }
                            }
                        });
                    }
                });
            }
        }
        else {
            console.log(timestamp('[hh:mm:ss]') + ' Aot compile finished');
            resolve();
        }
    }
}

function webpackBundle(allowLog) {
    return new Promise(function (resolve, reject) {
        console.log(timestamp('[hh:mm:ss]') + ' Webpack bundle running...');
        var srcfiles = appConfig.webpack.compileFiles;
        var totalFiles = 0;
        var countLoop = 0;

        for (var i = 0; i < srcfiles.length; i++) {
            glob(srcfiles[i], function (err, files) {
                countLoop++;
                totalFiles = totalFiles + files.length;
                for (var k = 0; k < files.length; k++) {
                    webpackCompile(appConfig.webpack.basePaths, appConfig.webpack.ignoreFiles, files[k], function () {
                        totalFiles = totalFiles - 1;
                        if (totalFiles === 0 && countLoop === srcfiles.length) {
                            console.log(timestamp('[hh:mm:ss]') + ' Webpack bundle finished');
                            resolve();
                        }
                    }, allowLog);
                }
            });
        }
        if (srcfiles.length === 0) {
            console.log(timestamp('[hh:mm:ss]') + ' Webpack bundle finished');
            resolve();
        }
    })
}

function compileLess(filePath) {
    var filePathFOrmatOS = filePath.replace(/\//g, "\\");

    gulp.src(filePath)
        .pipe($.sourcemaps.init())
        .pipe($.less({}))
        .pipe($.sourcemaps.write('.'))
        .pipe(gulp.dest(filePath.substring(0, filePath.lastIndexOf('/'))));

}

function compileAllLess() {
    for (var i = 0; i < appConfig.less.compile.tenantResource.files.length; i++) {
        glob(appConfig.less.compile.tenantResource.files[i], function (err, files) {
            for (var k = 0; k < files.length; k++) {
                compileLess(files[k]);
            }
        });
    }
}

function webpackCompile(basePaths, ignoreFiles, destPath, callBack, allowLog) {
    var isSelfResolveModule = false;
    var isCompile = false; // work around for teamcity not clean resource folder when rebuild
    if (!isMatchIgnoreFiles(ignoreFiles, destPath) && destPath.indexOf('.ngsummary.js') === -1) {
        let stream = fs.readFileSync(destPath, 'utf8');
        isCompile = stream.indexOf('webpackJsonp([0],{') !== 0;
        //isCompile = stream.indexOf('define(') !== 0;
        if (isCompile) {
            isSelfResolveModule = stream.indexOf('.OmniaControl({') > -1 || stream.indexOf('.OmniaAdminControl({') > -1 || stream.indexOf('.NgModule({') > -1 || stream.indexOf('.Component({') > -1
                || stream.indexOf('Extensibility_1.TemplateId') > -1 || stream.indexOf('Extensibility_1.OmniaControl') > -1 || stream.indexOf('<<webpack-resolve-module>>') > -1
        }
    }
    if (isCompile) {
        var namedModulesReplacements = [];
        for (var i = 0; i < basePaths.length; i++) {
            namedModulesReplacements.push({
                pattern: new RegExp(basePaths[i].replace(/\//g, "\\/") + "\\/", 'g'),
                replace: ""
            });
        }
        namedModulesReplacements.push({
            pattern: /\/node_modules/g,
            replace: ""
        });
        fs.appendFileSync(destPath, '\r\n //webpack-eof');
        destPath = "./" + destPath.replace(/[/\\*]/g, "/");
        var fileName = destPath.substring(destPath.lastIndexOf('/'));
        webpack(merge(require("./TaskRunner/config/webpack.config.js"), {
            entry: {
                'external': './TaskRunner/config/external.js',
                'app': destPath
            },
            externals: [
                function (context, request, callback) {
                    if (destPath.indexOf(request.replace("./", "")) === -1 || destPath.substring(destPath.indexOf(request.replace("./", "")), destPath.length) !== request.replace("./", "")) {
                        return callback(null, "__webpack_require__('" + request + "')");
                    }
                    callback();
                }
            ],
            //target: "node",
            cache: false,
            output: {
                //library: destPath,
                //libraryTarget: "amd",
                pathinfo: true,
                //path: '.',
                filename: destPath.replace("./", ''),
            },
            plugins: [
                new webpack.optimize.CommonsChunkPlugin({ name: 'external', filename: 'TaskRunner/config/external.bundle.js' }),
                new omniaPlugins.NamedModulesPlugin({
                    replacements: namedModulesReplacements
                }),
                new omniaPlugins.ReplaceBundlePlugin([
                    {
                        partten: /\/\/webpack-eof[\s\S]+}\,\[(.*?)\]/g,
                        replacement: function (match) {
                            return '}},[]'
                        }
                    },
                    {
                        partten: /define\((.*?)return webpackJsonp\(\[/i,
                        replacement: function (match) {
                            return match
                                .replace('define(', "define('" + destPath.replace("./", "") + "',")
                                .replace(/\[(.*?)\]/i, "[]") // remove dependency, will undo later
                                .replace(/function\((.*?){/i, "function(){") // remove dependency, will undo later
                                .replace(/__webpack_require__\(\'(.*?)'\)/g, function (module) {
                                    var requireModulePath = module
                                        .replace("__webpack_require__('", "")
                                        .replace("')", "")

                                    if (requireModulePath.indexOf(".") === 0) {
                                        var requireModulePath = path.resolve(path.dirname(destPath), requireModulePath);
                                        requireModulePath = requireModulePath.replace(/\\/g, "/");
                                        for (var i = 0; i < basePaths.length; i++) {
                                            requireModulePath = requireModulePath.replace(utils.root(basePaths[i]).replace(/\\/g, "/"), "")
                                        }
                                        requireModulePath = requireModulePath
                                            .replace(utils.root('TenantResources').replace(/\\/g, "/") + '/~/', "")
                                            .replace(utils.root('.').replace(/\\/g, "/") + '/~/', "")
                                            .replace(utils.root('.').replace(/\\/g, "/"), "")
                                            .trim("/");
                                        if (requireModulePath.indexOf("/") === 0)
                                            requireModulePath = requireModulePath.substring(1, requireModulePath.length);
                                        if (requireModulePath.indexOf("node_modules/") === 0)
                                            requireModulePath = requireModulePath.replace(/node_modules\//i, "");

                                    }
                                    return requireModulePath;

                                });
                        }
                    },
                    {
                        partten: /"use strict";[\s\S]+"use strict";/g,
                        replacement: '"use strict";'
                    },
                    {
                        partten: /\/\* no static exports found[\s\S]+function\(module, exports, __webpack_require__\) {/i,
                        replacement: function (match) { return "function(module, exports, __webpack_require__) {"; }
                    },
                    {
                        partten: new RegExp("__webpack_require__\\(\/\\*(.*)\\)", 'g'),
                        replacement: function (match) {
                            return match
                                .replace("/*!", "'")
                                .replace(/\*\/(.*?)\){1}/i, "')")
                                .replace(/ /g, "")
                                .replace(/'(.*?)'/i, function (requireModulePath) {
                                    requireModulePath = requireModulePath.replace(/'/g, "");
                                    if (requireModulePath.indexOf(".") === 0) {
                                        var requireModulePath = path.resolve(path.dirname(destPath), requireModulePath);
                                        requireModulePath = requireModulePath.replace(/\\/g, "/");
                                        for (var i = 0; i < basePaths.length; i++) {
                                            requireModulePath = requireModulePath.replace(utils.root(basePaths[i]).replace(/\\/g, "/"), "");
                                        }
                                        requireModulePath = requireModulePath
                                            .replace(utils.root('TenantResources').replace(/\\/g, "/") + '/~/', "")
                                            .replace(utils.root('.').replace(/\\/g, "/") + '/~/', "")
                                            .replace(utils.root('.').replace(/\\/g, "/"), "")
                                            .trim("/");
                                        if (requireModulePath.indexOf("/") === 0)
                                            requireModulePath = requireModulePath.substring(1, requireModulePath.length);
                                        if (requireModulePath.indexOf("node_modules/") === 0)
                                            requireModulePath = requireModulePath.replace(/node_modules\//i, "");
                                        if (destPath.indexOf(requireModulePath + '.js') > -1) {
                                            console.log('have recursive import self : ' + destPath);
                                        }
                                    }

                                    return "'" + requireModulePath + "'";
                                });
                        }
                    },
                    {
                        skip: !isSelfResolveModule,
                        partten: /webpackJsonp\(\[0\],{[\s\S]+":/g,
                        replacement: function (match) {
                            var moduleId = match.match(/"(.*?)"/)[1]
                            return "omfExecuteModules.push('" + moduleId + "'); ";

                            //return "require('" + destPath.replace("./", "") + "');\r\n" 
                            //        + "omfExecuteModules.push('" + moduleId + "'); ";
                        },
                        insertToEOF: true
                    },
                    //{
                    //    skip: isSelfResolveModule,
                    //    partten: /webpackJsonp\(\[0\],{[\s\S]+":/g,
                    //    replacement: function (match) {
                    //        return "require('" + destPath.replace("./", "") + "');";
                    //    },
                    //    insertToEOF: true
                    //},
                    {
                        partten: /;;/g,
                        replacement: ';'
                    },
                ])
            ]
        }), function (err, stats) {
            //if (callBack !== undefined)
            //    callBack();
            if (err) throw new gutil.PluginError("webpack", err);
            //gutil.log("[webpack]", stats.toString({
            //}));
        })
            .plugin("done", function () {
                if (allowLog)
                    console.log('bundle completed - ' + destPath);
                if (callBack !== undefined)
                    callBack();
            });
    }
    else {
        if (callBack !== undefined)
            callBack();
    }
}

function buildWebpackJsonpCallbackCustom() {
    return "function webpackJsonpCallback(chunkIds, moreModules, executeModules, forceExecute) { \r\n\
                var moduleId, chunkId, i = 0, resolves = [], result; \r\n\
                for(;i < chunkIds.length; i++) { \r\n\
                    chunkId = chunkIds[i];  \r\n\
                    if(installedChunks[chunkId])  \r\n\
                        resolves.push(installedChunks[chunkId][0]);  \r\n\
                    installedChunks[chunkId] = 0;  \r\n\
                } \r\n\
                for(moduleId in moreModules) { \r\n\
                    if(Object.prototype.hasOwnProperty.call(moreModules, moduleId)) { \r\n\
                        modules[moduleId.toLowerCase()] = moreModules[moduleId]; \r\n\
                    }\r\n\
                }\r\n\
                if(parentJsonpFunction) parentJsonpFunction(chunkIds, moreModules, executeModules, forceExecute); \r\n\
                while(resolves.length)\r\n\
                    resolves.shift()(); \r\n\
                if(executeModules) { \r\n\
                    for(i=0; i < executeModules.length; i++) { \r\n\
                        result = __webpack_require__(__webpack_require__.s = executeModules[i], forceExecute";
}

function buildWebpackResolveModuleCustom() {
    return "function __webpack_require__(moduleId, forceExecute) { \r\n\
                moduleId = moduleId.toLowerCase();\r\n\
                var execute = forceExecute || moduleId.indexOf('omnia/foundation/extensibility/directives/bootstrapcomponent.ngfactory') > -1;  \r\n\
                var originalModulePath = moduleId; \r\n\
                if(omfWebpackModulesMap[moduleId] !== undefined)moduleId = omfWebpackModulesMap[moduleId];\r\n\
                var foundModule = null;\r\n\
                if(moduleId.indexOf('./') !== 0)moduleId = './' + moduleId \r\n\
                var tempModuleId = moduleId;\r\n\
                if(moduleId.lastIndexOf('.js') !== moduleId.length -3)tempModuleId = moduleId + '.js'\r\n\
                if(execute !== true){\r\n\
                    foundModule = installedModules[tempModuleId]\r\n\
                    if(foundModule){if(webpackDebugMode && foundModule.l === false && webpackIgnoreModulesCircularDependency.indexOf(originalModulePath) === -1)\r\n\
                        console.log('Webpack warning circular dependency modules : ' + webpackModulesResolving.toString()); \r\n\
                        return foundModule.exports;}\r\n\
                    foundModule = installedModules[moduleId + '/index.js']\r\n\
                    if(foundModule){if(webpackDebugMode && foundModule.l === false && webpackIgnoreModulesCircularDependency.indexOf(originalModulePath) === -1)\r\n\
                        console.log('Webpack warning circular dependency modules : ' + webpackModulesResolving.toString()); \r\n\
                        return foundModule.exports;} \r\n\
                }\r\n\
                foundModule = modules[tempModuleId]\r\n\
                if(foundModule === undefined){ \r\n\
                    foundModule = modules[moduleId + '/index.js'];\r\n\
                    if(foundModule !== undefined) tempModuleId = moduleId + '/index.js' \r\n\
                }\r\n\
                if(foundModule === undefined){\r\n\
                    var isDisplayMessage = true; for (i = 0; i < omfWebpackIgnoreFindModules.length; i++) { if(moduleId.indexOf(omfWebpackIgnoreFindModules[i]) > -1){isDisplayMessage = false; break;} }    \r\n\
                    if(isDisplayMessage) \r\n\
                        console.log('Webpack not found module : ' + moduleId); return null;} \r\n\
                if(webpackDebugMode && webpackModulesResolving.indexOf(originalModulePath) === -1) webpackModulesResolving.push(originalModulePath);\r\n\
     		    var module = installedModules[tempModuleId] = { \r\n\
 			        i: tempModuleId, \r\n\
			        l: false, \r\n\
			        exports: {}}; \r\n\
     		    foundModule.call(module.exports, module, module.exports, __webpack_require__);\r\n\
    		    module.l = true;\r\n\
                if(webpackDebugMode && webpackModulesResolving.indexOf(originalModulePath) > -1) webpackModulesResolving.splice(webpackModulesResolving.indexOf(originalModulePath), 1);\r\n\
    		    return module.exports;"
}

function isMatchIgnoreFiles(ignoreFiles, filePath) {
    var isMatch = false
    for (var i = 0; i < ignoreFiles.length; i++) {
        if (filePath.indexOf(ignoreFiles[i]) > -1) {
            isMatch = true;
            return isMatch;
        }
    }

    return isMatch;
}

function enableRunAot() {
    var result = true;
    if (fs.existsSync(utils.root('./environment.json'))) {
        try {
            var config = require("./environment.json");
            if (config.Angular && config.Angular && config.Angular.AOT && config.Angular.AOT.RunOnBuild === false)
                result = false
        }
        catch (err) {
            console.log(err);
        }
    }

    return result;
}

function mergeConfig() {
    if (fs.existsSync(utils.root('./TaskRunner/config/app.extend.json'))) {
        try {
            var appConfigExtend = require("./TaskRunner/config/app.extend.json");
            if (appConfigExtend.webpack && appConfigExtend.webpack.basePath)
                appConfig.webpack.basePath = appConfigExtend.webpack.basePath
            if (appConfigExtend.webpack && appConfigExtend.webpack.compileFiles)
                appConfig.webpack.compileFiles = appConfigExtend.webpack.compileFiles
            if (appConfigExtend.webpack && appConfigExtend.webpack.ignoreFiles)
                appConfig.webpack.ignoreFiles = appConfigExtend.webpack.ignoreFiles
            if (appConfigExtend.webpack && appConfigExtend.webpack.watch)
                appConfig.webpack.watch = appConfigExtend.webpack.watch

            if (appConfigExtend.aot && appConfigExtend.aot.compileTscFiles)
                appConfig.aot.compileTscFiles = appConfigExtend.aot.compileTscFiles

            if (appConfigExtend.aot && appConfigExtend.aot.clean)
                appConfig.aot.clean = appConfigExtend.aot.clean
        }
        catch (err) {
            console.log('have an exception in app.extend.json : ' + err);
        }
    }
}



